function outputArgument = noiseBurstInFrequencyR2(commandString,currentDataStructure,randomHandleOption)
%   outBuffer = noiseBurstInFrequency(buffer,fs,f0,stretch)

%   F0 adaptive noise burst shaper-type1
%   noise is located on top of the excitation pulse
%   Designed and coded by Hideki Kawahara
%   24/Feb./2012
%   04/Mar./2012
%   26/Mar./2012 generalized
%   30/Oct./2014 bug fix for fixed rate operation
%   02/Aug./2015 fragility fix

switch commandString
    case 'initialize'
        currentDataStructure.buffer = zeros(currentDataStructure.fftLength,1);
        fs = currentDataStructure.samplingFrequency;
        if nargin > 2
            if isfield(randomHandleOption,'stretch')
                currentDataStructure.stretch = randomHandleOption.stretch;
            else
                currentDataStructure.stretch = 1;
            end;
        else
            currentDataStructure.stretch = 1;
        end;
        currentDataStructure.noiseBuffer = randn(round(1.1*currentDataStructure.temporalPositions(end)*fs),1);
        outputArgument = currentDataStructure;
    case 'fetch'
        fftl = currentDataStructure.fftLength;
        currentTime = currentDataStructure.eventLocations(currentDataStructure.eventCount);
        if ~isfield(currentDataStructure,'frameRateInSecond') % bug fix H.K. 2014/10/30
            halfLength = round(currentDataStructure.samplingFrequency/currentDataStructure.f0*currentDataStructure.stretch);
        else
            halfLength = round(currentDataStructure.samplingFrequency* ...
                currentDataStructure.frameRateInSecond*currentDataStructure.stretch); 
        end;
        w = hanning(halfLength*2+1).*currentDataStructure.noiseBuffer(...
            max(1,(-halfLength:halfLength)+round(currentTime*currentDataStructure.samplingFrequency)));
        outputArgument = currentDataStructure.buffer;
        indexTmp = fftl/2+1+(-halfLength:halfLength); % The following line fixes the issue. H.K. 2015.Aug.2
        outputArgument(min(currentDataStructure.fftLength,max(1,indexTmp))) = w/sqrt(currentDataStructure.stretch);
        outputArgument = fft(outputArgument);
end;
return;